;________________________________________________________________________
;
;                           G P S - E . A S M
;________________________________________________________________________
;
; PIC16F84/16C73A project by Byon Garrabrant  N6BG  byon@mail.com  Apr 98
;________________________________________________________________________

	LIST	P=PIC16F84
	include	"P16F84.INC"

;________________________________________________________________________

	cblock	H'20'			; system variables
		delayFlag
		phase
		sine_param
		freq_phase
		freq_add
		temp
		bit_timer
		new_freq
		FLAGS
		count
	endc

#define	SENT		FLAGS, 1

;__________________________________________________________________________

	ORG	0			; start code
	goto	main
;________________________________________________________________________

	ORG	H'0004'

	cblock				; interrupt variables
		intWsave
		intSTATUSsave
	endc

Timer_Interrupt
	movwf	intWsave		; save W
	swapf	STATUS, W		; save STATUS
	movwf	intSTATUSsave

	bcf	STATUS, RP0		; select bank 0 (working with TMR0)
	movlw	D'65'-2			; bump timer forward for every 65 ins
	subwf	TMR0, F			; update TMR0 for next int

	movfw	freq_add		; move the phase forward by add;
	addwf	freq_phase, F
	movfw	freq_phase		; get the sine
	andlw	H'7F'			; convert 0->255 to 0->127
	btfsc	freq_phase, 6		; if >= 64
	 sublw	D'128'			; subtract from 128 (0->63->64->1)
	call	Get_Quarter_Sine	; lookup quarter sine table
	btfsc	freq_phase, 7		; if converted above
	 sublw	0			; invert answer
	andlw	H'F0'
	movwf	temp			; divide by 16
	swapf	temp, F

	movfw	PORTB
	andlw	H'F0'
	iorwf	temp, W
	movwf	PORTB			; output it

	decfsz	bit_timer, F
	 goto	Timer_End

	movfw	new_freq
	movwf	freq_add
	bsf	SENT			; ready for the next bit

	movlw	D'32'			; how long to run this tone
	movwf	bit_timer

Timer_End
	bcf	INTCON, T0IF		; enable TMR0 int
	swapf	intSTATUSsave, W	; restore STATUS
	movwf	STATUS
	swapf	intWsave, F		; restore W
	swapf	intWsave, W
	retfie				; return from interrupt

Timer_Interrupt_Enable
	bsf	STATUS, RP0		; Select Bank 1
	movlw	b'11011000'
	movwf	OPTION_REG		; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0
	bsf	INTCON, GIE		; enable ints
	bsf	INTCON, T0IE		; enableTMR0int
	return

Timer_Interrupt_Disable
	bcf	INTCON, T0IE		; disableTMR0int
	bsf	STATUS, RP0		; Select Bank 1
	movlw	b'11010111'		; divide TMR0 by 256
	movwf	OPTION_REG		; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0
	return

;________________________________________________________________________

Get_Quarter_Sine	; w enters 0 to 64, w exits 128 to 255
	addwf	PCL, F
	dt	D'128',D'131',D'134',D'137',D'140',D'144',D'147',D'150'
	dt	D'153',D'156',D'159',D'162',D'165',D'168',D'171',D'174'
	dt	D'177',D'179',D'182',D'185',D'188',D'191',D'193',D'196'
	dt	D'199',D'201',D'204',D'206',D'209',D'211',D'213',D'216'
	dt	D'218',D'220',D'222',D'224',D'226',D'228',D'230',D'232'
	dt	D'234',D'235',D'237',D'239',D'240',D'241',D'243',D'244'
	dt	D'245',D'246',D'248',D'249',D'250',D'250',D'251',D'252'
	dt	D'253',D'253',D'254',D'254',D'254',D'255',D'255',D'255'
	dt	D'255'

;________________________________________________________________________

Get_Packet_Byte
	addwf	PCL, F
Packet_Start
	dt	'C'<<1, 'Q'<<1, ' '<<1, ' '<<1, ' '<<1, ' '<<1, H'60'
	dt	'N'<<1, '6'<<1, 'B'<<1, 'G'<<1, ' '<<1, ' '<<1, H'61'
	dt	H'03',H'F0'
Packet_End
#define	Packet_Length	Packet_End-Packet_Start

;________________________________________________________________________

	cblock
		CRC_lo
		CRC_hi
	endc	

CRC_Init
	movlw	H'FF'			; crc inits to 0xFFFF
	movwf	CRC_hi
	movwf	CRC_lo
	return

CRC_Calc
	movlw	1
	skpnc
	 xorwf	CRC_lo, F		; crc ^= d & 1;

	clrc
	rrf	CRC_hi, F
	rrf	CRC_lo, F
	skpc
	 return
	movlw	H'08'
	xorwf	CRC_lo, F
	movlw	H'84'
	xorwf	CRC_hi, F		; crc = ( crc & 1 ) ? ( crc >> 1 ) ^ POLY : ( crc >> 1 );
	return

;________________________________________________________________________

	cblock
		bit_history
		sending_char
		sending_char_count
		flag_count
	endc	

Send_Data_Bit
	btfss	SENT			; wait for last bit to be sent
	 goto	Send_Data_Bit

	rlf	bit_history, F		; store the last 8 bits sent
	rrf	bit_history, W		; restore the bit

	movfw	new_freq		
	skpc				; if sending a 0
	 sublw	D'23'			;  change frequency	D'8' + D'15'
	movwf	new_freq

	bcf 	SENT			; next bit is ready

	comf	bit_history, W		; check the last 5 bits
	andlw	H'1F'
 	skpz				; if not all 1s
 	 return				; return

	clrc				; send a transparent 0 bit
	goto	Send_Data_Bit			
 	return				; return

;________________________________________________________________________

Send_Data_Byte
	movwf	sending_char
	movlw	8
	movwf	sending_char_count
Send_Data_Byte_Loop
	rrf	sending_char, W		; get bit for CRC
	call	CRC_Calc
	rrf	sending_char, F		; get bit for sending
	call	Send_Data_Bit
	decfsz	sending_char_count, F
	 goto	Send_Data_Byte_Loop
	return 
;________________________________________________________________________

Send_CRC_Byte
	movwf	sending_char
	movlw	8
	movwf	sending_char_count
Send_CRC_Byte_Loop
	rrf	sending_char, F		; get bit for sending
	call	Send_Data_Bit
	decfsz	sending_char_count, F
	 goto	Send_CRC_Byte_Loop
	return 
;________________________________________________________________________

Send_Flag_Byte
	movlw	H'7E'
	movwf	sending_char
	movlw	8
	movwf	sending_char_count
Send_Flag_Byte_Loop
	rrf	sending_char, F		; get bit for sending
 	call	Send_Data_Bit
	clrf	bit_history		; no bit stuffing for flags
	decfsz	sending_char_count, F
	 goto	Send_Flag_Byte_Loop
	return 
;________________________________________________________________________

Send_Flags
	movwf	flag_count
Send_Flag_Loop
	call	Send_Flag_Byte
	decfsz	flag_count, F
	 goto	Send_Flag_Loop
	return
;________________________________________________________________________

 if 0

Send_Header

	call	CRC_Init
	clrw				 ; location 0 holds header length

	movlw	HEADER_OFFSET
	movwf	header_index
	movwf	header_length

	call	Get_Message_Byte
	addwf	header_length, F

	incf	header_index, F


header_loop:
	movfw	header_index

	call	Get_Message_Byte

	call	Send_Data_Byte

	incf	header_index, F
	movfw	header_index
	xorwf	header_length, W
	skpz
	 goto	header_loop
	return

;________________________________________________________________________

Send_Title
	clrf	title_index
	movlw	'$'
	call	Send_Data_Byte

title_loop:
	movfw	title_index
	call	Get_Title
	call	Send_Data_Byte
	incf	title_index, F
	movfw	title_index
	xorlw	TITLE_LEN
	skpz
	 goto	title_loop
	return

;________________________________________________________________________

Send_Message
	clrf	message_index

message_loop:
	movfw	message_index
	addlw	MESSAGE_BASE
	movwf	FSR
	movfw	INDF
	call	Send_Data_Byte

	incf	message_index, F
	movfw	message_index
	xorwf	message_length, W
	skpz
	 goto	message_loop
	return
 endif
;________________________________________________________________________

Send_CRC:
	movlw	H'FF'
	xorwf	CRC_lo, F
	xorwf	CRC_hi, F
	movfw	CRC_lo
	call	Send_CRC_Byte
	movfw	CRC_hi
	call	Send_CRC_Byte
	return
;________________________________________________________________________

#define		MAX_DATA_LEN		D'20'
	cblock
		received_char
		data_index
		mesg : MAX_DATA_LEN	; GPS message RAM
	endc

	cblock
		packet_index
	endc

Send_Packet
	call	Timer_Interrupt_Enable

	movlw	D'30'
	call	Send_Flags
	call	CRC_Init
	clrf	packet_index

Send_Packet_Loop
	movfw	packet_index
	call	Get_Packet_Byte
	call	Send_Data_Byte
	incf	packet_index, F
	movlw	Packet_Length
	xorwf	packet_index, W
	skpz
	 goto	Send_Packet_Loop

	clrf	packet_index
Send_Message_Loop
	movfw	packet_index
	addlw	mesg			; adjust to mesg data space
	movwf	FSR			; set the FSR
	movfw	INDF			; get it from RAM
	call	Send_Data_Byte
	incf	packet_index, F
	movfw	packet_index
	xorwf	data_index, W
	skpz
	 goto	Send_Message_Loop

	call	Send_CRC

	movlw	D'2'
	call	Send_Flags

	call	Timer_Interrupt_Disable

	return
;________________________________________________________________________

Send_Serial

	clrf	packet_index
Send_SMessage_Loop
	movfw	packet_index
	addlw	mesg			; adjust to mesg data space
	movwf	FSR			; set the FSR
	movfw	INDF			; get it from RAM
	call	serialTransmit
	incf	packet_index, F
	movfw	packet_index
	xorwf	data_index, W
	skpz
	 goto	Send_SMessage_Loop

	movlw	H'0D'
	call	serialTransmit
	movlw	H'0A'
	call	serialTransmit
	return
;________________________________________________________________________

Get_Serial
	call	serialReceive		; check for incoming serial data
	andlw	H'FF'			; did we get something?
	skpnz				; if not
	 return				;  leave

	movwf	received_char		; save the serial data

	movfw	data_index		; get the data index
	addlw	mesg			; adjust to mesg data space
	movwf	FSR			; set the FSR

	movfw	received_char		; get the received char
	movwf	INDF			; store it in RAM

	incf	data_index, F		; bump to next data char
	movfw	data_index		; check the title length
	xorlw	MAX_DATA_LEN		; have we received entire title
	skpnz				; if so,
	 goto	get_data_done		;  done

	movfw	received_char		; was the last char 
	xorlw	D'13'			; a star
	skpz				; if not so,
	 return				;  return

get_data_done
	call	Send_Packet		; send it
	call	Send_Serial		; send it
	clrf	data_index		; reset title index
	return				; done

;________________________________________________________________________

	include	"SERIAL.ASM"		; include serial code
;________________________________________________________________________

main
	clrf	data_index		; reset title index

	bsf	STATUS, RP0		; Select Bank 1

	movlw	B'11110000'		; make audio an output
	movwf	PORTB

	bcf	STATUS, RP0		; Select Bank 0

	clrf	freq_phase

	movlw	D'8'
	movlw	D'15'
	movwf	new_freq
	movwf	freq_add

	movlw	D'1'
	movwf	bit_timer

	clrf	INTCON
	call	Timer_Interrupt_Disable

	call	serialInit

	clrf	count

mainLoop
	call	Get_Serial
	goto	mainLoop		; do it again

	call	Send_Packet
	call	Send_Serial

	goto	mainLoop		; do it again

	END
;________________________________________________________________________
