;________________________________________________________________________
;
;                           M I C - E . A S M
;________________________________________________________________________
;
; PIC16C73 project by Byon Garrabrant  N6BG  byon@mail.com  Jan 98
;________________________________________________________________________

	LIST	P=PIC16C73A
	include	"P16C73A.INC"

	__CONFIG _HS_OSC & _CP_OFF & _BODEN_OFF & _PWRTE_OFF & _WDT_OFF

;________________________________________________________________________

	cblock	H'20'			; system variables
		tempCount
	endc

	ORG	0			; start code
	goto	main
;________________________________________________________________________

; PORTA PIN 0 - AD0
; PORTA PIN 1 - AD1 & MSG bit 3
; PORTA PIN 2 - AD2
; PORTA PIN 3 - AD3 Period Pot
; PORTA PIN 4 - Auto/Manual & Carrier Detect
; PORTA PIN 5 - AD4
; PORTB PIN 0 - MIC PTT
; PORTB PIN 1 - MSG bit 0
; PORTB PIN 2 - MSG bit 1
; PORTB PIN 3 - MSG bit 2
; PORTB PIN 4 - PATH bit 0
; PORTB PIN 5 - PATH bit 1
; PORTB PIN 6 - PATH bit 2
; PORTB PIN 7 - PATH bit 3
; PORTC PIN 0 - RADIO PTT (OUTPUT)
; PORTC PIN 1 - AUDIO OUT (OUTPUT)
; PORTC PIN 2 - EEPROM DATA OUT (OUTPUT)
; PORTC PIN 3 - EEPROM DATA IN 
; PORTC PIN 4 - EEPROM CLOCK (OUTPUT)
; PORTC PIN 5 - EEPROM CS (OUTPUT)
; PORTC PIN 6 - SERIAL IN
; PORTC PIN 7 - SERIAL OUT (OUTPUT)

#define	RADIO_PTT	PORTC, 0
#define	MIC_PTT		PORTB, 0
#define	BANK1		H'7F'

;________________________________________________________________________

	cblock				; serial Sending variables
		intWsave
		intSTATUSsave
;		intRate			; D'64'-2 ins clocks per int
		intFreqPhase
		intFreqAdd
		intFreqNext
		intToneTimer
		intFlags
		intTemp
	endc
#define		FREQ_SENT	intFlags, 0

	ORG	H'0004'

interrupt
	movwf	intWsave		; save W
	swapf	STATUS, W		; save STATUS
	movwf	intSTATUSsave

;	bcf	STATUS, RP0		; select bank 0 (working with TMR0)
;	movfw  	intRate			; bump timer by
;	subwf	TMR0, F			; update TMR0 for next int

	movfw	intFreqAdd		; move the phase forward by add;
	addwf	intFreqPhase, F
	movfw	intFreqPhase

	call	GetSine			; comment out for sawtooth

	movwf	intTemp
	rrf	intTemp, F
	rrf	intTemp, W

	call	pwmSet

	decfsz	intToneTimer, F
	 goto	interruptEnd

	movfw	intFreqNext
	movwf	intFreqAdd
	bsf	FREQ_SENT		; ready for the next bit
	movlw	D'16'			; how long to run this tone
	movwf	intToneTimer

interruptEnd
	bcf	INTCON, T0IF		; enable TMR0 int
	swapf	intSTATUSsave, W	; restore STATUS
	movwf	STATUS
	swapf	intWsave, F		; restore W
	swapf	intWsave, W
	retfie				; return from interrupt

interruptEnable
	bsf	STATUS, RP0		; Select Bank 1
	movlw	b'01011000'
	movwf	BANK1 & OPTION_REG	; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0

	bsf	INTCON, T0IE		; enableTMR0int
	bsf	INTCON, GIE		; enable ints
	return

interruptDisable
	bcf	INTCON, T0IE		; disableTMR0int
	bcf	INTCON, GIE		; disable ints
	return

;________________________________________________________________________

Get_Header_Byte
	addwf	PCL, F
HEADER_START
	dt	HEADER_END-HEADER_START
	dt	'C'<<1, 'Q'<<1, ' '<<1, ' '<<1, ' '<<1, ' '<<1, H'60'
	dt	'N'<<1, '6'<<1, 'B'<<1, 'G'<<1, ' '<<1, ' '<<1, H'7E'
	dt	'W'<<1, 'I'<<1, 'D'<<1, 'E'<<1, ' '<<1, ' '<<1, H'61'
	dt	H'03',H'F0'
	dt	"Byon Here!"
HEADER_END
;________________________________________________________________________

functionTable
	swapf	PORTB, W
	andlw	H'0F'
	addwf	PCL, F
	goto	sendA2D			; 0   w+
	goto	echoKeys		; 1   e+
	goto	writeEEPROM		; 2   s+
	goto	readEEPROM		; 3   n+
	goto	pwmOut			; 4   w
	goto	sine1200		; 5   e
	goto	sine2200		; 6   s
	goto	sineToggle		; 7   n
	goto	sine2200		; 8   7
	goto	sine2200		; 9   6
	goto	sine2200		; A   5
	goto	sineToggle		; B   4
	goto	Packet_Send_Flags	; C   3
	goto	sendPacket		; D   2
	goto	$			; E   1
	goto	$			; F   0

;________________________________________________________________________

GetQuarterSine	; w enters 0 to 64, w exits 128 to 255
	addwf	PCL, F
	dt	D'128',D'131',D'134',D'137',D'140',D'144',D'147',D'150'
	dt	D'153',D'156',D'159',D'162',D'165',D'168',D'171',D'174'
	dt	D'177',D'179',D'182',D'185',D'188',D'191',D'193',D'196'
	dt	D'199',D'201',D'204',D'206',D'209',D'211',D'213',D'216'
	dt	D'218',D'220',D'222',D'224',D'226',D'228',D'230',D'232'
	dt	D'234',D'235',D'237',D'239',D'240',D'241',D'243',D'244'
	dt	D'245',D'246',D'248',D'249',D'250',D'250',D'251',D'252'
	dt	D'253',D'253',D'254',D'254',D'254',D'255',D'255',D'255'
	dt	D'255'
;________________________________________________________________________

	cblock				; serial Sending variables
		sine_param
	endc

GetSine		; w enters 0 to 255, w exits 1 to 255
	movwf	sine_param		; save w
	andlw	H'7F'			; convert 0->255 to 0->127
	btfsc	sine_param, 6		; if >= 64
	 sublw	D'128'			; subtract from 128 (0->63->64->1)
	call	GetQuarterSine		; lookup quarter sine table
	btfsc	sine_param, 7		; if converted above
	 sublw	0			; invert answer
	return
;________________________________________________________________________


serialInit
	bsf	RCSTA,SPEN		; enable Serial Port

	bsf	STATUS, RP0		; Select Bank 1
	bcf	BANK1 & PORTC, 6	; make output
	bcf	BANK1 & TXSTA,SYNC 	; Async mode
	movlw	D'64'			; 20M/64(64+1) = 4800
	movwf	BANK1 & SPBRG		; 4800 baud
	bcf	BANK1 & TXSTA,BRGH	; low speed
	bsf	BANK1 & TXSTA,TXEN	; enable TX
	bcf	BANK1 & PIE1,TXIE	; no TX interrupts
	bcf	BANK1 & PIE1,RCIE 	; no RX interrupts
	bcf	STATUS, RP0		; Select Bank 0

	bsf	RCSTA,CREN		; enable RX
	return
	
serialTransmit
	btfss	PIR1,TXIF		; wait for completion of last byte
	 goto	serialTransmit
	movwf	TXREG			; start the transmitter
	return				; return

serialReceiveReady
	btfss	PIR1,RCIF		; is data ready?
	 retlw	0			;  not ready
	retlw	1			; ready

serialReceive
	btfss	PIR1,RCIF		; wait for reception of data
	 goto	serialReceive
	btfsc	RCSTA,OERR		; did we overrun?
	 goto	serialReceiveError	;  handle error
	btfsc	RCSTA,FERR		; did we frame error?
	 goto	serialReceiveError	;  handle error
	movfw	RCREG			; read the data
	return				; return

serialReceiveError
	movfw	RCREG			; read the data
	bcf	RCSTA,CREN 		; disable RX - clears the error
	bsf	RCSTA,CREN 		; enable RX
	return				; return
;________________________________________________________________________

	cblock				; serial Sending variables
		sendHexValue
	endc

sendSpace
	movlw	' '
	goto	serialTransmit

sendCRLF
	movlw	H'0D'
	call	serialTransmit
	movlw	H'0A'
	goto	serialTransmit

sendHex
	movwf	sendHexValue
	swapf	sendHexValue, W
	call	sendHexNibble
	movfw	sendHexValue

sendHexNibble
	andlw	H'0F'
	sublw	9
	skpc
	 addlw	-7
	sublw	'0'+9
	goto	serialTransmit

;________________________________________________________________________

#define	AD0	B'10000001'
#define	AD1	B'10001001'
#define	AD2	B'10010001'
#define	AD3	B'10011001'
#define	AD4	B'10100001'

a2dInit
	bsf	STATUS, RP0		; Select Bank 1
	clrf	BANK1 & ADCON1		; all channels A/D
	bcf	STATUS, RP0		; Select Bank 0
	clrf	ADRES			; clear A/D result
	return

a2dRead	
	movwf	ADCON0

	movlw	D'20'			; delay for >50 instructions (10 us)
	movwf	tempCount
	decfsz	tempCount, F
	 goto	$-1

	bsf	ADCON0, GO		; start A/D conversion

	btfsc	ADCON0, GO		; wait for A/D completion
	 goto	$-1
	movfw	ADRES			; get A/D result
	return				; return it
;________________________________________________________________________

#define	EEPROM_DO	PORTC, 2
#define	EEPROM_DI	PORTC, 3
#define	EEPROM_CLK	PORTC, 4
#define	EEPROM_CS	PORTC, 5

#define	EEPROM_READ_OPCODE		B'11000000'
#define	EEPROM_WRITE_OPCODE		B'10100000'
#define	EEPROM_ENABLE_OPCODE		B'10000000'
#define	EEPROM_WRITE_ENABLE		B'11000000'
#define	EEPROM_WRITE_DISABLE		B'00000000'

	cblock				; serial Sending variables
		eepromOpcode
		eepromAddress
		eepromDataLo
		eepromDataHi
	endc

eepromTransferBit	macro	f
	bcf	EEPROM_DO		; set up data out
	rlf	f, W
	skpnc
	 bsf	EEPROM_DO
	bsf	EEPROM_CLK		; clock
	nop
	bcf	EEPROM_CLK
	clrc				; read data in
	btfsc	EEPROM_DI
	 setc
	rlf	f, F			; shift
	endm

eepromInit
	bsf	STATUS, RP0		; Select Bank 1
	bcf	EEPROM_DO		; data out is an OUTPUT
	bcf	EEPROM_CLK		; clock is an OUTPUT
	bcf	EEPROM_CS		; chip select is an OUTPUT
	bcf	STATUS, RP0		; Select Bank 0
	bcf	EEPROM_DO		; clear data out
	bcf	EEPROM_CLK		; clear clock
	bcf	EEPROM_CS		; clear chip select
	return

eepromSendOpcodeAddress
	eepromTransferBit	eepromOpcode	; send opcode
	eepromTransferBit	eepromOpcode	; send opcode
	eepromTransferBit	eepromOpcode	; send opcode
	eepromTransferBit	eepromAddress	; send address
	eepromTransferBit	eepromAddress	; send address
	eepromTransferBit	eepromAddress	; send address
	eepromTransferBit	eepromAddress	; send address
	eepromTransferBit	eepromAddress	; send address
	eepromTransferBit	eepromAddress	; send address
	eepromTransferBit	eepromAddress	; send address
	eepromTransferBit	eepromAddress	; send address
	return

eepromTransferData
	eepromTransferBit	eepromDataHi	; transfer data
	eepromTransferBit	eepromDataHi	; transfer data
	eepromTransferBit	eepromDataHi	; transfer data
	eepromTransferBit	eepromDataHi	; transfer data
	eepromTransferBit	eepromDataHi	; transfer data
	eepromTransferBit	eepromDataHi	; transfer data
	eepromTransferBit	eepromDataHi	; transfer data
	eepromTransferBit	eepromDataHi	; transfer data
eepromTransferDataLo
	eepromTransferBit	eepromDataLo	; transfer data
	eepromTransferBit	eepromDataLo	; transfer data
	eepromTransferBit	eepromDataLo	; transfer data
	eepromTransferBit	eepromDataLo	; transfer data
	eepromTransferBit	eepromDataLo	; transfer data
	eepromTransferBit	eepromDataLo	; transfer data
	eepromTransferBit	eepromDataLo	; transfer data
	eepromTransferBit	eepromDataLo	; transfer data
	return

eepromRead
	bsf	EEPROM_CS		; set chip select
	movlw	EEPROM_READ_OPCODE
	movwf	eepromOpcode
	call	eepromSendOpcodeAddress	; send read opcode and address
	call	eepromTransferDataLo	; get data
	bcf	EEPROM_CS		; clear chip select
	return				; return it

eepromReadMore
	bsf	EEPROM_CS		; set chip select
	call	eepromTransferDataLo	; get data
	bcf	EEPROM_CS		; clear chip select
	return				; return it

eepromWrite
	bsf	EEPROM_CS		; set chip select
	movlw	EEPROM_WRITE_OPCODE
	movwf	eepromOpcode
	call	eepromSendOpcodeAddress	; send read opcode and address
	call	eepromTransferData	; get data
	bcf	EEPROM_CS		; clear chip select (start write)

	nop
	bsf	EEPROM_CS		; set chip select (begin polling)
	btfss	EEPROM_DI		; wait for write to complete
	 goto	$-1
	bcf	EEPROM_CS		; clear chip select
	return				; return

eepromWriteEnable
	bsf	EEPROM_CS		; set chip select
	movlw	EEPROM_ENABLE_OPCODE
	movwf	eepromOpcode
	movlw	EEPROM_WRITE_ENABLE
	movwf	eepromAddress
	call	eepromSendOpcodeAddress	; send erase/write enable opcode
	bcf	EEPROM_CS		; clear chip select
	return				; return

eepromWriteDisable
	bsf	EEPROM_CS		; set chip select
	movlw	EEPROM_ENABLE_OPCODE
	movwf	eepromOpcode
	movlw	EEPROM_WRITE_DISABLE
	movwf	eepromAddress
	call	eepromSendOpcodeAddress	; send erase/write disable opcode
	bcf	EEPROM_CS		; clear chip select
	return				; return

;________________________________________________________________________

#define	PWM	PORTC, 1

pwmInit
	bcf	PWM			; make PWM an output
	bsf	STATUS, RP0		; Select Bank 1
	bcf	PWM			; make PWM an output
	movlw	H'3F'			; 63 = 78.12Khz
	movwf	BANK1 & PR2
	bcf	STATUS, RP0		; Select Bank 0
	bcf	PWM			; make PWM an output

	clrf	T2CON			; set no scaling
	bsf	T2CON, TMR2ON		; set Timer2 ON
	return

pwmSet
	andlw	H'3F'			
	movwf	CCPR2L			; set PWM rate
	movlw	B'00001100'		; PWM mode on
	movwf	CCP2CON
	return

;________________________________________________________________________

	cblock				; main loop variables
		CRC_hi
		CRC_lo
	endc

CRC_Init
	movlw	H'FF'			; crc inits to 0xFFFF
	movwf	CRC_hi
	movwf	CRC_lo
	return

CRC_Calc
	movlw	1
	skpnc
	 xorwf	CRC_lo, F		; crc ^= d & 1;
	clrc
	rrf	CRC_hi, F
	rrf	CRC_lo, F
	skpc
	 return
	movlw	H'08'
	xorwf	CRC_lo, F
	movlw	H'84'
	xorwf	CRC_hi, F		; crc = ( crc & 1 ) ? ( crc >> 1 ) ^ POLY : ( crc >> 1 );
	return
;________________________________________________________________________

	cblock				; main loop variables
		PacketBitHistory
		PacketChar
		PacketBitCount
		PacketFlagCount
		PacketHeaderIndex
		PacketHeaderLength
	endc

Packet_Send_Bit
	btfss	FREQ_SENT		; lock and wait for last bit to be sent
	 goto	$-1

	rlf	PacketBitHistory, F	; store the last 8 bits sent
	rrf	PacketBitHistory, W	; restore the bit

	movfw	intFreqNext		
	skpc				; if sending a 0
	 sublw	D'16'+D'29'		;  change frequency	D'16' + D'29'
	movwf	intFreqNext
	
	bcf 	FREQ_SENT		; next bit is ready

	comf	PacketBitHistory, W	; check the last 5 bits
	andlw	H'1F'
 	skpz				; if not all 1s
 	 return				; return

	clrc				; send a transparent 0 bit
	goto	Packet_Send_Bit

;________________________________________________________________________

Packet_Send_Data_Byte
	movwf	PacketChar
	movlw	8
	movwf	PacketBitCount
Packet_Send_Data_Byte_Loop
	rrf	PacketChar, W		; get bit for CRC
	call	CRC_Calc
	rrf	PacketChar, F		; get bit for sending
	call	Packet_Send_Bit
	decfsz	PacketBitCount, F
	 goto	Packet_Send_Data_Byte_Loop
	return 

Packet_Send_CRC_Byte
	movwf	PacketChar
	movlw	8
	movwf	PacketBitCount
Packet_Send_CRC_Byte_Loop
	rrf	PacketChar, F		; get bit for sending
	call	Packet_Send_Bit
	decfsz	PacketBitCount, F
	 goto	Packet_Send_CRC_Byte_Loop
	return 

Packet_Send_Flag_Byte
	movlw	H'7E'
	movwf	PacketChar
	movlw	8
	movwf	PacketBitCount
Packet_Send_Flag_Byte_Loop
	rrf	PacketChar, F		; get bit for sending
	call	Packet_Send_Bit
	clrf	PacketBitHistory	; no bit stuffing for flags
	decfsz	PacketBitCount, F
	 goto	Packet_Send_Flag_Byte_Loop
	return 
;________________________________________________________________________

Packet_Send_Flags
	movwf	PacketFlagCount
Packet_Send_Flag_Loop
	call	Packet_Send_Flag_Byte
	decfsz	PacketFlagCount, F
	 goto	Packet_Send_Flag_Loop
	return

Packet_Send_Header
	call	CRC_Init
	clrf	PacketHeaderIndex
	movfw	PacketHeaderIndex
	call	Get_Header_Byte
	movwf	PacketHeaderLength
	incf	PacketHeaderIndex, F
Packet_Send_Header_Loop
	movfw	PacketHeaderIndex
	call	Get_Header_Byte
	call	Packet_Send_Data_Byte
	incf	PacketHeaderIndex, F
	movfw	PacketHeaderIndex
	xorwf	PacketHeaderLength, W
	skpz
	 goto	Packet_Send_Header_Loop
	return

Packet_Send_CRC:
	movlw	H'FF'
	xorwf	CRC_lo, F
	xorwf	CRC_hi, F
	movfw	CRC_lo
	call	Packet_Send_CRC_Byte
	movfw	CRC_hi
	call	Packet_Send_CRC_Byte
	return
;________________________________________________________________________

Packet_Send
	movlw	D'16'		
	movwf	intFreqNext
	call	interruptEnable
	bsf	RADIO_PTT
	movlw	D'30'
	call	Packet_Send_Flags	; send initial flags
	call	Packet_Send_Header
	call	Packet_Send_CRC
	movlw	D'2'
	call	Packet_Send_Flags	; send 2 final flags
	bcf	RADIO_PTT
	call	interruptDisable
	return
;________________________________________________________________________

	cblock				; main loop variables
		mainCount
	endc

main
	bsf	STATUS, RP0		; Select Bank 1
	bcf	BANK1 & OPTION_REG,7	; enable PORTB PULLUPS
	bcf	BANK1 & RADIO_PTT	; RADIO_PTT is an OUTPUT
	bcf	STATUS, RP0		; Select Bank 0

	call	serialInit
	call	a2dInit
	call	eepromInit
	call	pwmInit

	clrf	mainCount

mainLoop
	goto	functionTable

echoKeys
	call	serialReceiveReady
	andlw	H'FF'
	bz	mainLoop

	call	serialReceive
	addlw	1
	call	serialTransmit

	goto	mainLoop		; do it again

sendA2D
	movlw	AD0
	call	a2dRead
	call	sendHex
	call	sendSpace

	movlw	AD1
	call	a2dRead
 	call	sendHex
 	call	sendSpace

	movlw	AD2
	call	a2dRead
 	call	sendHex
 	call	sendSpace

	movlw	AD3
	call	a2dRead
  	call	sendHex
  	call	sendSpace

	movlw	AD4
    	call	a2dRead
    	call	sendHex
    	call	sendSpace

	call	sendCRLF

	goto	mainLoop		; do it again

writeEEPROM
	call	serialReceiveReady
	andlw	H'FF'
	bz	mainLoop

	call	serialReceive
	movwf	eepromDataHi
	movwf	eepromDataLo

	call	eepromWriteEnable	; enable before setting address because it clears it

	incf	mainCount, F
	movfw	mainCount
	movwf	eepromAddress

	call	eepromWrite
	call	eepromWriteDisable	
	goto	mainLoop		; do it again

readEEPROM
	incf	mainCount, F
	movfw	mainCount
	movwf	eepromAddress
	call	eepromRead
	movfw	eepromDataLo
	movwf	eepromDataHi
	call	eepromReadMore

	movfw	mainCount
    	call	sendHex
    	call	sendSpace

	movfw	eepromDataHi
    	call	sendHex
	movfw	eepromDataLo
    	call	sendHex

    	call	sendSpace

	movfw	eepromDataHi
	call	serialTransmit
	movfw	eepromDataLo
	call	serialTransmit
	call	sendCRLF

	goto	mainLoop		; do it again

pwmOut
	call	serialReceiveReady
	andlw	H'FF'
	bz	mainLoop

	call	serialReceive
	andlw	H'3F'
	movwf	mainCount

	movfw	mainCount
	call	pwmSet

	movfw	mainCount
    	call	sendHex
    	call	sendSpace

	goto	mainLoop		; do it again

sine1200
;	movlw	D'256'-2
;	movwf	intRate
	movlw	D'16'
	movwf	intFreqNext
	bsf	RADIO_PTT
	call	interruptEnable
	goto	$

sine2200
;	movlw	D'256'-2
;	movwf	intRate
	movlw	D'29'
	movwf	intFreqNext
	bsf	RADIO_PTT
	call	interruptEnable
	goto	$

sineToggle
	bsf	RADIO_PTT
	call	interruptEnable
sineToggleLoop
	movlw	D'29'
	movwf	intFreqNext
	bcf	FREQ_SENT
	btfss	FREQ_SENT
	 goto	$-1

	movlw	D'16'
	movwf	intFreqNext
	bcf	FREQ_SENT
	btfss	FREQ_SENT
	 goto	$-1

	goto	sineToggleLoop

sendPacket
	call	Packet_Send
	goto	mainLoop		; do it again

	END
;________________________________________________________________________
