;__________________________________________________________________________
;
; Packet.asm
; Byon Garrabrant  N6BG  byon@mail.com
;__________________________________________________________________________

	LIST	P=PIC16F84
	include	"P16F84.INC"
	radix	hex
;__________________________________________________________________________

#define		MAX_DATA_LEN		D'31'

	cblock	0x0C			; start of ram

		FLAGS			; generic flags / bits
		index			; used to count header, message, etc.
		length
		temp

		intWsave		; interrupt flags
		intSTATUSsave

		freq_add		; packet sending flags
		freq_phase
		new_freq		; next freq to send
		bit_timer		; number of ints to continue tone
		CRC_lo			; packet CRC
		CRC_hi
		sending_char		; character bits to send
		sending_char_count	; number of bits left to send
		bit_history		; last bits sent (for bit stuffing)

		last_TMR0		; timing flags
		sec_counter		; for counting 10s of seconds
		beacon_counter		; 10s of seconds left till beacon

		mesg : MAX_DATA_LEN	; GPS message RAM
		message_length

		fix_timer
		unlock_countL
		unlock_countH
	endc

;__________________________________________________________________________

#define		MESSAGE_BASE		mesg
#define		flag_count		length
#define		sine_nibble		temp
#define		sine_param		temp
#define		header_index		index
#define		header_length		length
#define		title_index		length
#define		message_index		length
#define		data_index		length
#define		received_char		temp
#define		menu_index		index

;__________________________________________________________________________

#define	SENT		FLAGS, 1
#define	WAITING		FLAGS, 2
#define	MENU_MODE	FLAGS, 3
#define	RECV_DATA	FLAGS, 4
#define	LAST_CALL	FLAGS, 5
#define	PTT		PORTA, 2

#define	FIX		PORTB, 5
#define	TEST		PORTB, 4

;__________________________________________________________________________

	org	H'2100'			; start of EEPROM data

EEPROM_DATA

NUM_FLAGS_START
;	dt	64		; 64 flags
	dt	D'100'		; 64 flags
BEACON_RATE_START
	dt	D'30'		; b e 30 = 5 min
;	dt	D'1'		; b e 30 = 5 min
HEADER_START
	dt	HEADER_END-HEADER_START
	dt	'C'<<1, 'Q'<<1, ' '<<1, ' '<<1, ' '<<1, ' '<<1, H'60'
	dt	'N'<<1, '6'<<1, 'B'<<1, 'G'<<1, ' '<<1, ' '<<1, H'7E'; 61
;	dt	'R'<<1, 'E'<<1, 'L'<<1, 'A'<<1, 'Y'<<1, ' '<<1, H'60'
	dt	'W'<<1, 'I'<<1, 'D'<<1, 'E'<<1, ' '<<1, ' '<<1, H'60'
	dt	'W'<<1, 'I'<<1, 'D'<<1, 'E'<<1, ' '<<1, ' '<<1, H'60'
;	dt	'W'<<1, 'I'<<1, 'D'<<1, 'E'<<1, ' '<<1, ' '<<1, H'60'
	dt	'W'<<1, 'I'<<1, 'D'<<1, 'E'<<1, ' '<<1, ' '<<1, H'61'
	dt	H'03',H'F0'
HEADER_END

#define	HEADER_OFFSET		(HEADER_START - EEPROM_DATA)
#define	NUM_FLAGS_OFFSET	(NUM_FLAGS_START - EEPROM_DATA)
#define	BEACON_RATE_OFFSET	(BEACON_RATE_START - EEPROM_DATA)
;__________________________________________________________________________

	ORG	H'0000'
	goto	packetInit		; Power On Reset Vector
;________________________________________________________________________

	ORG	H'0004'

Timer_Interrupt
	movwf	intWsave		; save W
	swapf	STATUS, W		; save STATUS
	movwf	intSTATUSsave

	bcf	STATUS, RP0		; select bank 0 (working with TMR0)
	movlw	D'52'-2			; bump timer forward for
	subwf	TMR0, F			; update TMR0 for next int

	movfw	freq_add		; move the phase forward by add;
	addwf	freq_phase, F

	movfw	freq_phase
	call	Get_Sine
	andlw	H'F0'
	movwf	sine_nibble
	swapf	sine_nibble, F
	clrw
	movfw	PORTB
	andlw	H'F0'
	iorwf	sine_nibble, W
	movwf	PORTB

	decfsz	bit_timer, F
	 goto	timer_end

	movfw	new_freq
	movwf	freq_add
	bsf	SENT			; ready for the next bit

	movlw	D'16'			; how long to run this tone
	movwf	bit_timer

timer_end

	bcf	INTCON, T0IF		; enable TMR0 int
	swapf	intSTATUSsave, W	; restore STATUS
	movwf	STATUS
	swapf	intWsave, F		; restore W
	swapf	intWsave, W
	retfie				; return from interrupt

;__________________________________________________________________________

Get_Quarter_Sine	; w enters 0 to 64, w exits 128 to 255
	addwf	PCL, F
	dt	D'128',D'131',D'134',D'137',D'140',D'144',D'147',D'150'
	dt	D'153',D'156',D'159',D'162',D'165',D'168',D'171',D'174'
	dt	D'177',D'179',D'182',D'185',D'188',D'191',D'193',D'196'
	dt	D'199',D'201',D'204',D'206',D'209',D'211',D'213',D'216'
	dt	D'218',D'220',D'222',D'224',D'226',D'228',D'230',D'232'
	dt	D'234',D'235',D'237',D'239',D'240',D'241',D'243',D'244'
	dt	D'245',D'246',D'248',D'249',D'250',D'250',D'251',D'252'
	dt	D'253',D'253',D'254',D'254',D'254',D'255',D'255',D'255'
	dt	D'255'
;__________________________________________________________________________

Get_Title
	addwf	PCL, F
	dt	"GPGLL,"
#define	TITLE_LEN	6

;________________________________________________________________________

	org	h'100'

_Get_Menu_Text
	addwf	PCL, F
Menu_Text_Start
Menu_Title_Text
	dt	H'0C'
	dt	"PIC-Pac Configuration Menu", H'0D', H'0A', 0
Menu_Dest_Text
	dt	H'0D', H'0A', "1)Destination:", 0
Menu_Call_Text
	dt	H'0D', H'0A', "2)Callsign:", 0
Menu_Digi_Text
	dt	H'0D', H'0A', "3)Digipeaters:", 0
Menu_Beacon_Text
 	dt	H'0D', H'0A', "4)Beacon Every:", 0
Menu_Beacon2_Text
	dt	" (x10 seconds)", 0
Menu_Done_Text
	dt	H'0D', H'0A', "5)Exit Menu", 0
Menu_Option_Text
	dt	H'0D', H'0A', H'0D', H'0A', "Choose Option (1-5) >", 0
Menu_Exit_Text
	dt	H'0D', H'0A', "Exiting Menu", H'0D', H'0A', 0

#define	Menu_Title	(Menu_Title_Text - Menu_Text_Start)
#define	Menu_Call	(Menu_Call_Text - Menu_Text_Start)
#define	Menu_Dest	(Menu_Dest_Text - Menu_Text_Start)
#define	Menu_Digi	(Menu_Digi_Text - Menu_Text_Start)
#define	Menu_Beacon	(Menu_Beacon_Text - Menu_Text_Start)
#define	Menu_Beacon2	(Menu_Beacon2_Text - Menu_Text_Start)
#define	Menu_Done	(Menu_Done_Text - Menu_Text_Start)
#define	Menu_Option	(Menu_Option_Text - Menu_Text_Start)
#define	Menu_Exit	(Menu_Exit_Text - Menu_Text_Start)


Get_Menu_Text
	bsf	PCLATH, 0
	call	_Get_Menu_Text
	bcf	PCLATH, 0
	return
;________________________________________________________________________

	include	"SERIAL.ASM"		; include LCD code
;________________________________________________________________________

Get_Message_Byte
EERead
	andlw	H'3F'			; only 64 EE data bytes
	movwf	EEADR			; set the address
	bsf	STATUS, RP0		; select bank 1
	bsf	EECON1, RD		; EEPROM read
	bcf	STATUS, RP0		; select bank 0
	movfw	EEDATA			; get data
	return

;________________________________________________________________________

EEWrite
	andlw	H'3F'			; only 64 EE data bytes
	movwf	EEADR			; set the address, assume data is set
	bsf	STATUS, RP0		; select bank 1
	bcf	INTCON, GIE		; disable ints
	bsf	EECON1, WREN		; enable writes
	movlw	H'55'
	movwf	EECON2			; write 55
	movlw	H'AA'
	movwf	EECON2			; write AA
	bsf	EECON1, WR		; EEPROM write
	bcf	EECON1, WREN		; disable writes
	bsf	INTCON, GIE		; enable ints
EEWriteWait
	btfsc	EECON1, WR		; test writing bit
	 goto	EEWriteWait		; keep waiting
	bcf	STATUS, RP0		; select bank 0
	return
;________________________________________________________________________

Get_Sine		; w enters 0 to 255, w exits 1 to 255


	movwf	sine_param		; save w
	andlw	H'7F'			; convert 0->255 to 0->127
	btfsc	sine_param, 6		; if >= 64
	 sublw	D'128'			; subtract from 128 (0->63->64->1)
	call	Get_Quarter_Sine	; lookup quarter sine table
	btfsc	sine_param, 7		; if converted above
	 sublw	0			; invert answer
	return

;________________________________________________________________________

CRC_Init
	movlw	H'FF'			; crc inits to 0xFFFF
	movwf	CRC_hi
	movwf	CRC_lo
	return

CRC_Calc
	movlw	1
	skpnc
	 xorwf	CRC_lo, F		; crc ^= d & 1;

	clrc
	rrf	CRC_hi, F
	rrf	CRC_lo, F
	skpc
	 return
	movlw	H'08'
	xorwf	CRC_lo, F
	movlw	H'84'
	xorwf	CRC_hi, F		; crc = ( crc & 1 ) ? ( crc >> 1 ) ^ POLY : ( crc >> 1 );
	return

;________________________________________________________________________

Send_Data_Bit
	btfss	SENT			; wait for last bit to be sent
	 goto	Send_Data_Bit

	rlf	bit_history, F		; store the last 8 bits sent
	rrf	bit_history, W		; restore the bit

	movfw	new_freq		
	skpc				; if sending a 0
	 sublw	D'45'			;  change frequency	D'16' + D'29'
	movwf	new_freq
	
	bcf 	SENT			; next bit is ready

	comf	bit_history, W		; check the last 5 bits
	andlw	H'1F'
 	skpz				; if not all 1s
 	 return				; return

	clrc				; send a transparent 0 bit
	goto	Send_Data_Bit			

;________________________________________________________________________

Send_Data_Byte
	movwf	sending_char
	movlw	8
	movwf	sending_char_count
Send_Data_Byte_Loop
	rrf	sending_char, W		; get bit for CRC
	call	CRC_Calc
	rrf	sending_char, F		; get bit for sending
	call	Send_Data_Bit
	decfsz	sending_char_count, F
	 goto	Send_Data_Byte_Loop
	return 
;________________________________________________________________________

Send_CRC_Byte
	movwf	sending_char
	movlw	8
	movwf	sending_char_count
Send_CRC_Byte_Loop
	rrf	sending_char, F		; get bit for sending
	call	Send_Data_Bit
	decfsz	sending_char_count, F
	 goto	Send_CRC_Byte_Loop
	return 
;________________________________________________________________________

Send_Flag_Byte:
	movlw	H'7E'
	movwf	sending_char
	movlw	8
	movwf	sending_char_count
Send_Flag_Byte_Loop
	rrf	sending_char, F		; get bit for sending
 	call	Send_Data_Bit
	clrf	bit_history		; no bit stuffing for flags
	decfsz	sending_char_count, F
	 goto	Send_Flag_Byte_Loop
	return 
;________________________________________________________________________

Send_Flags
	movwf	flag_count
Send_Flag_Loop
	call	Send_Flag_Byte
	decfsz	flag_count, F
	 goto	Send_Flag_Loop
	return
;________________________________________________________________________

Send_Header:

	call	CRC_Init
	clrw				 ; location 0 holds header length

	movlw	HEADER_OFFSET
	movwf	header_index
	movwf	header_length

	call	Get_Message_Byte
	addwf	header_length, F

	incf	header_index, F


header_loop:
	movfw	header_index

	call	Get_Message_Byte

	call	Send_Data_Byte

	incf	header_index, F
	movfw	header_index
	xorwf	header_length, W
	skpz
	 goto	header_loop
	return

;________________________________________________________________________

Send_Title:
	clrf	title_index
	movlw	'$'
	call	Send_Data_Byte

title_loop:
	movfw	title_index
	call	Get_Title
	call	Send_Data_Byte
	incf	title_index, F
	movfw	title_index
	xorlw	TITLE_LEN
	skpz
	 goto	title_loop
	return

;________________________________________________________________________

Send_Message:
	clrf	message_index

message_loop:
	movfw	message_index
	addlw	MESSAGE_BASE
	movwf	FSR
	movfw	INDF
	call	Send_Data_Byte

	incf	message_index, F
	movfw	message_index
	xorwf	message_length, W
	skpz
	 goto	message_loop
	return

;________________________________________________________________________

Send_CRC:
	movlw	H'FF'
	xorwf	CRC_lo, F
	xorwf	CRC_hi, F
	movfw	CRC_lo
	call	Send_CRC_Byte
	movfw	CRC_hi
	call	Send_CRC_Byte
	return

;________________________________________________________________________

Packet_Send_Locked
	clrf	unlock_countH		; reset unlock count
	clrf	unlock_countL		; reset unlock count

Packet_Send_Try_Unlock
	call	Serial_Get		; try to unlock
	btfss	RECV_DATA		; if we are not recving data
	 goto	Packet_Send		; we are not locked, send
	decfsz	unlock_countL, F	; keep trying to unlock
	 goto	Packet_Send_Try_Unlock		
	decfsz	unlock_countH, F  	; keep trying to unlock
	 goto	Packet_Send_Try_Unlock		
					; we failed to unlock
	bsf	WAITING			; we are waiting for the $
	bcf	RECV_DATA		; we are not recving data
	return				; just dont send it

Packet_Send
;	btfsc	RECV_DATA		; if we are not recving data
;	 goto	Packet_Send_Locked	; we are not locked

	bsf	STATUS, RP0		; Select Bank 1
	movlw	b'11011000'
	movwf	OPTION_REG		; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0

	bsf	PTT			; press PTT

	bsf	INTCON, T0IE		; enableTMR0int

	movlw	NUM_FLAGS_OFFSET
	call	EERead
	call	Send_Flags		; send initial flags
	call	Send_Header
	call	Send_Title
	call	Send_Message
	call	Send_CRC
	movlw	D'2'
	call	Send_Flags		; send 2 flags
	bcf	INTCON, T0IE		; disableTMR0int

	bcf	PTT
	bsf	STATUS, RP0		; Select Bank 1
	movlw	b'11010111'		; divide TMR0 by 256
	movwf	OPTION_REG		; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0

	bsf	WAITING			; we are waiting for the $
	bcf	RECV_DATA		; we are not recving data

	return

;________________________________________________________________________

Serial_Get

	call	serialReceive		; check for incoming serial data
	andlw	H'FF'			; did we get something?
	skpnz				; if not
	 return				;  leave

	movwf	received_char		; save the serial data
	clrf	unlock_countH		; reset unlock count while receving


	btfsc	MENU_MODE		; if in menu mode
	 goto	Menu_Input		;  handle it

	btfss	WAITING			; we got something.  Are we waiting for $?
	 goto	not_waiting		; if not, jump ahead

bad_char
	bsf	WAITING			; we are waiting for the $
	bcf	RECV_DATA		; we are not recving data

	movfw	received_char		; compare the received char
	xorlw	D'27'			; to ESC
	skpnz				; if equal
	 goto	Menu_Start

	movfw	received_char		; save the serial data
	xorlw	'$'			; we are waiting for $.  Did we get it?
	skpz				; if not
	 return				;  leave

	bcf	WAITING			; we are no longer waiting for the $
	bcf	RECV_DATA		; we are in title check mode
	clrf	title_index		; reset title index
	return				; done

not_waiting				; we are not waiting for $
	btfsc	RECV_DATA		; if we are recving data,
	 goto	get_data		;  go do it

	movfw	title_index		; get the next title char
	call	Get_Title		
	xorwf	received_char, W	; compare to the received char
	skpz				; if they are different
	 goto	bad_char		;  we are back to waiting for $

	incf	title_index, F		; bump to next title char
	movfw	title_index		; check the title length
	xorlw	TITLE_LEN		; have we received entire title
	skpz				; if not
	 return				;  leave

	bsf	RECV_DATA		; we got the title, now in data aquire mode

	movlw	D'120'
	movwf	fix_timer

	clrf	data_index		; reset the data_idx
	return				; done

get_data				; we are receiving the data

	movfw	data_index		; get the data index
	addlw	MESSAGE_BASE		; adjust to mesg data space
	movwf	FSR			; set the FSR

	movfw	received_char		; get the received char
	movwf	INDF			; store it in RAM

	incf	data_index, F		; bump to next data char
	movfw	data_index		; check the title length
	xorlw	MAX_DATA_LEN		; have we received entire title
	skpnz				; if so,
	 goto	get_data_done		;  done

	movfw	received_char		; was the last char 
	xorlw	'*'			; a star
	skpz				; if not so,
	 return				;  return

get_data_done
	movfw	data_index		; set the message_length
	movwf	message_length
	bsf	WAITING			; we got the message, now in waiting mode
	bcf	RECV_DATA		; we are not recving data
	return				; done

;________________________________________________________________________

Menu_Send
	movwf	menu_index
Menu_Send_Loop:
	movfw	menu_index
	call	Get_Menu_Text
	andlw	H'FF'
	skpnz
	 return
	call	serialTransmit		; echo the char
	incf	menu_index, F
	goto	Menu_Send_Loop

Menu_Send_Call
	movwf	header_index
	movlw	6
	movwf	header_length
	bcf	LAST_CALL
Menu_Send_Call_Loop:
	movfw	header_index
	call	Get_Message_Byte	; get the callsign char
	movwf	temp
	clrc
	rrf	temp, F			; shift the char
	movfw	temp
	xorlw	' '
	bz	dont_send
	movfw	temp
	call	serialTransmit		; send the char
dont_send
	incf	header_index, F
	decfsz	header_length, F
	 goto	Menu_Send_Call_Loop

Menu_Send_SSID
	movfw	header_index
	call	Get_Message_Byte	; get the callsign char
	andlw	H'1F'
	movwf	temp
	clrc
	rrf	temp, F
	skpnc
	 bsf	LAST_CALL
	tstf	temp
	skpnz	
	 return
	movlw	'-'
	call	serialTransmit		; send the - char
	movlw	D'10'
	subwf	temp, W
	bc	two_digits
	movfw	temp
	addlw	'0'
	call	serialTransmit		; send the digit
	return

two_digits
	movlw	'1'
	call	serialTransmit		; send the tens digit
	movlw	D'10'
	subwf	temp, W
	addlw	'0'
;	movfw	temp
;	sublw	D'10'

	call	serialTransmit		; send the digit
	return


Menu_Start
	bsf	MENU_MODE		; we are now in menu mode
	bcf	RECV_DATA		; we are not recving data

	movlw	Menu_Title
	call	Menu_Send

	movlw	Menu_Dest
	call	Menu_Send
	movlw	D'1'
	call	Menu_Send_Call

	movlw	Menu_Call
	call	Menu_Send
	movlw	D'8'
	call	Menu_Send_Call

	movlw	Menu_Digi
	call	Menu_Send

	btfsc	LAST_CALL
	 goto	Menu_Digi_Done
	movlw	D'15'
	call	Menu_Send_Call

	btfsc	LAST_CALL
	 goto	Menu_Digi_Done
	movlw	','
	call	serialTransmit		; send the digit
	movlw	D'22'
	call	Menu_Send_Call

Menu_Digi_Done
	movlw	Menu_Beacon
	call	Menu_Send

	movlw	'2'
	call	serialTransmit		; send the digit

	movlw	Menu_Beacon2
	call	Menu_Send

	movlw	Menu_Done
	call	Menu_Send

	movlw	Menu_Option
	call	Menu_Send
	return

Menu_Input:
	call	serialTransmit		; echo the char

	movfw	received_char		; compare the received char
	xorlw	'5'			; to 5.  
	bz	Menu_Quit		; if same, quit

	movfw	received_char		; compare the received char
	xorlw	'$'			; to $.  
	bz	Menu_Quit		; if same, quit

	movfw	received_char		; compare the received char
	xorlw	H'0D'			; to CR.  
	bz	Menu_EOL		; if same, handle EOL
	return

Menu_EOL
	movlw	H'0A'
	call	serialTransmit
	return

Menu_Quit
	bcf	MENU_MODE		; we are now out of menu mode
	bsf	WAITING			; we are waiting for the $
	bcf	RECV_DATA		; we are not recving data
	movlw	Menu_Exit		; print the exit message
	call	Menu_Send
	return
	
;________________________________________________________________________

Timer_Update		; dec sec_counter each time TMR0 wraps

	movfw	last_TMR0		; find difference of last_TMR0
	subwf	TMR0, W			; and current TMR0
	addwf	last_TMR0, F		; update last_TMR0 with difference
	skpc				; if no wrap
	 return				;  leave

	tstf	fix_timer
	bz	no_fix
	bsf	FIX
	decf	fix_timer, F
	goto	fix_done
no_fix
	bcf	FIX
fix_done

 	decfsz	sec_counter, F		; dec it and if not zero
	 return 			; leave

	movlw	D'153'			; reset to every 10 secounds
	movwf	sec_counter

	decfsz	beacon_counter, F	; dec it and if not zero
	 return 			; leave

	movlw	BEACON_RATE_OFFSET
	call	EERead
	movwf	beacon_counter

	call	Packet_Send		; time to send

	return
;________________________________________________________________________

packetInit
	call	serialInit

	bcf	PTT			; release PTT
	bsf	STATUS, RP0		; Select Bank 1
	movlw	B'11000000'
	movwf	PORTB
	bcf	PTT			; make PTT an output
	movlw	b'11010111'		; divide TMR0 by 256
	movwf	OPTION_REG		; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0
	bcf	PTT			; release PTT
	clrf	INTCON
	bsf	INTCON, GIE		; enable ints

	movlw	D'16'			; set initial frequency
	movwf	new_freq		

	bsf	WAITING			; we are waiting for the $
	bcf	RECV_DATA		; we are not recving data
	bcf	MENU_MODE		; we are not in menu mode

	movlw	MAX_DATA_LEN		; initialize GPS RAM
	movwf	message_index
	movlw	MESSAGE_BASE
	movwf	FSR

message_init
	movlw	'0'
	movwf	INDF
	incf	FSR, F
	decfsz	message_index, F
	 goto	message_init

	movlw	1
	movwf	message_length
	movwf	sec_counter		; reset second counter

	movlw	BEACON_RATE_OFFSET
	call	EERead
	movwf	beacon_counter		; reset beacon counter

	clrf	fix_timer

	bcf	FIX

packetLoop
	call	Packet_Send
	goto	packetLoop
	call	Serial_Get
	call	Timer_Update
	goto	packetLoop
;__________________________________________________________________________

	END

todo
  smart gpgll receive	
  menu mode
  separate modules
  serial diodes	 	(hardware)
  manual mode 		(hardware needed)
  use carrier detect	(hardware needed)



$GPRMC,083036,A,3610.999,N,11516.337,W,001.6,326.6,221197,013.7,E*6F
$GPRMC,072004,A,5124.164,N,00039.253,W,021.7,028.9,180496,004.1,W*78 

$GPGGA,083036,3610.999,N,11516.337,W,1,04,2.4,810.7,M,-25.2,M,,*72
$GPGGA,072005,5124.165,N,00039.252,W,1,04,3.2,-19.8,M,47.8,M,,*4C 

$GPGLL,3610.999,N,11516.337,W,083037,A*3F
$GPGLL,5124.153,N,00039.257,W,072003,A*31 


from the net

$GPGLL,5124.153,N,00039.257,W,072003,A*31 
$GPRMC,072004,A,5124.164,N,00039.253,W,021.7,028.9,180496,004.1,W*78 
$GPGGA,072005,5124.165,N,00039.252,W,1,04,3.2,-19.8,M,47.8,M,,*4C 
$GPGSA,A,3,,,07,09,,,26,,,,,,3.2,3.2,1.0*39 
$GPGSV,2,1,07,02,27,053,50,05,19,212,32,07,30,098,51,09,68,275,32*73 
$GPGSV,2,2,07,21,20,317,32,23,38,276,40,26,61,155,51,,,,*40 
$PGRME,74.7,M,150.0,M,167.6,M*18 
$GPGLL,5124.168,N,00039.249,W,072005,A*30 

from me

$GPRMC,083036,A,3610.999,N,11516.337,W,001.6,326.6,221197,013.7,E*6F
$GPRMB,A,,,,,,,,,,,,V*71
$GPGGA,083036,3610.999,N,11516.337,W,1,04,2.4,810.7,M,-25.2,M,,*72
$GPGSA,A,3,04,,,,10,,24,30,,,,,4.2,2.4,3.4*35
$GPGSV,2,1,08,04,14,049,39,05,67,185,00,06,30,299,00,09,06,199,00*75
$GPGSV,2,2,08,10,51,120,43,17,07,233,00,24,55,035,45,30,56,300,36*74
$PGRME,57.5,M,79.6,M,98.2,M*12
$GPGLL,3610.999,N,11516.337,W,083037,A*3F
$PGRMZ,2660,f,3*29
$PGRMM,WGS 84*06
$GPBOD,,T,,M,,*47
$GPRTE,1,1,c,0*07
