;________________________________________________________________________
;
;                           S E R I A L . A S M
;________________________________________________________________________
;
; Software serial routines for PIC16C84 
; by Byon Garrabrant  N6BG  byon@netcom.com  Mar 97
;________________________________________________________________________

	cblock				; variables
		serialDelayCount
		serialCount
		serialData
	endc
;________________________________________________________________________

#define		SERIAL_IN	PORTA, 1
#define		SERIAL_OUT	PORTA, 0

; DELAY_TIME should be (ClockSpeed/4/SerialSpeed-14)/3
; 4MHz 9600 baud would be (4000000/4/9600-14)/3 = 30
; 4MHz 2400=134 4800=65 9600=30 19200=13 38400=4 57600=1

; 10Mhz  4800=169
DELAY_TIME	equ	D'169'

;________________________________________________________________________

serialDelayThird
	movlw	DELAY_TIME/3		
	movwf	serialDelayCount	; only count 1/3
	goto	serialDelayLoop		; use routine below
serialDelay
	movlw	DELAY_TIME		
	movwf	serialDelayCount	; Count down from DELAY_TIME
	nop				; Helps with high speed
serialDelayLoop
	decfsz	serialDelayCount, F	; Countdown
	 goto	serialDelayLoop		; keep counting
	return				; done
;________________________________________________________________________

serialInit
	bsf	STATUS, RP0		; Select Bank 1
	bsf	SERIAL_IN		; Setup input
	bcf	SERIAL_OUT		; Setup output
	bcf	STATUS, RP0		; Select Bank 0
	return

;________________________________________________________________________

serialTransmit
	movwf	serialData		; save the outgoing byte
	movlw	D'10'
	movwf	serialCount		; set the bit count
	clrc				; prep for start bit
serialTransmitLoop
	bcf	SERIAL_OUT		; assume send low
	skpc				; if sending 0
	 bsf	SERIAL_OUT		; send high
	call	serialDelay		; delay
	setc				; prep for stop bit
	rrf	serialData, F		; roll serialIn bits	
	decfsz	serialCount, F		; if not complete
	 goto	serialTransmitLoop	; loop back for more
	return				; all done
;________________________________________________________________________

serialReceive
	btfss	SERIAL_IN		; Check for Start bit
	 retlw	0			; not receiving now
	movlw	D'9'			
	movwf	serialCount		; gonna read 9 bits
	call	serialDelayThird	; skip third a bit
serialReceiveLoop
	clrc				; assume clear carry
	btfss	SERIAL_IN		; if receiving 0
	 setc				; set carry
	rrf	serialData, F		; roll serialData bits	
	call	serialDelay		; delay
	nop				; keep timing w/ transmit
	decfsz	serialCount, F		; if not complete
	 goto	serialReceiveLoop	; loop back for more
	movfw	serialData		; return the data
	return
;________________________________________________________________________

